import io.undertow.Undertow;
import io.undertow.server.HttpHandler;
import io.undertow.server.HttpServerExchange;
import io.undertow.util.Headers;
import io.undertow.websockets.WebSocketConnectionCallback;
import io.undertow.websockets.core.AbstractReceiveListener;
import io.undertow.websockets.core.BufferedTextMessage;
import io.undertow.websockets.core.WebSocketChannel;
import io.undertow.websockets.core.WebSockets;
import io.undertow.websockets.spi.WebSocketHttpExchange;

import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Scanner;
import java.util.stream.Stream;

import static io.undertow.Handlers.*;

public class CuisineHTTPServer extends Thread
{
    static int webPagePort = 8080;

    static int webSocketPort = 42069;
    static String webSocketHost = "192.168.0.24";

    static int tcpSocketPort = 6868;
    static String HOST = "0.0.0.0" ;


    private static CuisineTCPServer fCuisineTCPServer;

    public void startServer(String iHostName, int iWebPagePort, int iWebSocketPort, int iTcpSocketPort)
    {
        webSocketHost = iHostName;
        webPagePort = iWebPagePort;
        webSocketPort = iWebSocketPort;
        tcpSocketPort = iTcpSocketPort;
        initResources();

        //Create CuisineTCPServers
        fCuisineTCPServer = new CuisineTCPServer(tcpSocketPort);
        CuisineWebSocketMessageHandler vWebSocketMessageHandler         = new CuisineWebSocketMessageHandler( fCuisineTCPServer );
        CuisineWebSocketConnectionHandler vWebSocketConnectionHandler   = new CuisineWebSocketConnectionHandler( vWebSocketMessageHandler );

        //Start HTTP Server : serve webpage with JS that calls the websocket
        Undertow WebPageServer = Undertow.builder()
                .addHttpListener(webPagePort, HOST)
                .setHandler(
                        new CuisineResponseHandler("ws://"+webSocketHost+":"+webSocketPort+"/data")
                            ).build();
        WebPageServer.start();

        System.out.println("WebPageServer started");
        //Start websocket
        Undertow WebSocketServer = Undertow.builder()
            .addHttpListener(webSocketPort, HOST)
            .setHandler(
                path()
                    .addPrefixPath("/data",websocket(vWebSocketConnectionHandler))
            )
            .build();
        WebSocketServer.start();
        System.out.println("WebSocketServer started");

    }

    static String sHtml_ASCII_title;
    static String sHtml_CSS;
    static void initResources()
    {
        try {
            sHtml_ASCII_title = new Scanner(CuisineHTTPServer.class.getResourceAsStream("html_ascii_title"), "UTF-8").useDelimiter("\\A").next();
            sHtml_CSS = new Scanner(CuisineHTTPServer.class.getResourceAsStream("style.css"), "UTF-8").useDelimiter("\\A").next();
        }
        catch(Exception vex)
        {
            vex.printStackTrace();
            sHtml_ASCII_title = "Galère les resources";
        }
    }

    private class CuisineResponseHandler implements HttpHandler
    {
        private String fWebSocketConnectionUri;

        public CuisineResponseHandler(String iWebSocketConnectionUri)
        {
            super();

            fWebSocketConnectionUri = iWebSocketConnectionUri;
        }
        @Override
        public void handleRequest(HttpServerExchange exchange) throws Exception {
            //Serve webpage
            exchange.getResponseHeaders()
                    .put(Headers.CONTENT_TYPE, "text/html")
                    .put(Headers.CONTENT_ENCODING,"utf-8");
            exchange.getResponseSender().send(getHTMLFrontPage(fWebSocketConnectionUri));
        }
    }

    private class CuisineWebSocketConnectionHandler implements WebSocketConnectionCallback
    {
        private CuisineWebSocketMessageHandler fWebSocketMessageHandler;

        protected CuisineWebSocketConnectionHandler(CuisineWebSocketMessageHandler iWebSocketMessageHandler)
        {
        fWebSocketMessageHandler = iWebSocketMessageHandler;
        }

        @Override
        public void onConnect(WebSocketHttpExchange exchange, WebSocketChannel channel) {
            channel.getReceiveSetter().set(fWebSocketMessageHandler);
            channel.resumeReceives();
        }
    }

    private class CuisineWebSocketMessageHandler extends AbstractReceiveListener {

        private CuisineTCPServer fTCPServer;

        protected CuisineWebSocketMessageHandler( CuisineTCPServer iTCPServer)
        {

            fTCPServer = iTCPServer;
        }

        protected void onFullTextMessage(WebSocketChannel iChannel, BufferedTextMessage iTextMessage)
        {
            //Handle message on webSocket
            WebSockets.sendText(String.valueOf(fTCPServer.getCurrentTemperature()), iChannel, null);
        }
    }


    static String getHTMLFrontPage(String iWsUri)
    {
        StringBuilder vBuilder = new StringBuilder();
        vBuilder
            .append("<head>")
            .append("   <meta charset=\"UTF-8\">")
            .append("   <style>")
            .append(sHtml_CSS)
            .append("   </style>")
            .append("</head>")
            .append("<body>")
            .append("   <div class=\"centered\">")
            .append(sHtml_ASCII_title)
            .append("   </div>")
            .append("   <h1 id=\"tempValue\">None</h1>")
            .append("   <script>")
            .append("       var socket=new WebSocket(\""+iWsUri+"\");\n")
            .append("       let connected=false;")
            .append("       let view=document.getElementById(\"tempValue\");")
            .append("       socket.onerror=function(error){connected=false;view.innerHTML=\"oups :(\";};")
            .append("       socket.onmessage=function(event){view.innerHTML=event.data+\"&deg;C\"};")
            .append("       socket.onopen=function(e){")
            .append("           connected=true;")
            .append("           callServer();")
            .append("       };")
            .append("       var callServer=function(){")
            .append("           socket.send(\"coucou\");")
            .append("       };")
            .append("       let timer=setInterval(callServer,100);")
            .append("   </script>")
            .append("</body>")
            .append("")

            ;
        return vBuilder.toString();
    }
}
